unit Multithread_Ex;

{ Graphics32 Example }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Graphics32, StdCtrls;

type
  TPointF = record
    X, Y: Single;
  end;

  TPaintThread = class(TThread)
  protected
    Image: TImage32;
    P1, P2: TPointF;
    V1, V2: TPointF;
    W, H: Integer;
    ColorTime: Single;
    Color: TColor32;
    procedure NextFrame;
  public
    constructor Create(AnImage: TImage32);
    procedure Changed;
    procedure Execute; override;
  end;

  TForm1 = class(TForm)
    Image: TImage32;
    Button1: TButton;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    Thread1: TPaintThread;
    Thread2: TPaintThread;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

function PointF(X, Y: Single): TPointF;
begin
  Result.X := X;
  Result.Y := Y;
end;

{ TPaintThread }

constructor TPaintThread.Create(AnImage: TImage32);
begin
  Image := AnImage;
  W := Image.Bitmap.Width;
  H := Image.Bitmap.Height;
  Randomize;
  ColorTime := Random;

  { generate initial positions }
  P1 := PointF(Random(W), Random(H));
  P2 := PointF(Random(W), Random(H));

  { generate initial velocities }
  V1 := PointF(Random - 0.5, Random - 0.5);
  V2 := PointF(Random - 0.5, Random - 0.5);

  inherited Create(False);
end;

procedure TPaintThread.Changed;
begin
  Image.Invalidate;
end;

procedure TPaintThread.Execute;
begin
  FreeOnTerminate := True;
  Priority := tpLower;
  while not (Terminated or Application.Terminated) do
  begin
    with Image.Bitmap do
    begin
      NextFrame;
      Lock;
      try
        DrawLineFS(P1.X, P1.Y, P2.X, P2.Y, Color);
      finally
        Unlock;
      end;
    end;
    Synchronize(Changed);
    Application.ProcessMessages;
  end
end;

procedure TPaintThread.NextFrame;

  procedure AdvancePoint(var P, V: TPointF);
  begin
    { apply velocity }
    P.X := P.X + V.X;
    P.Y := P.Y + V.Y;

    { reflect from walls }
    if P.X < 0 then
    begin
      P.X := 0;
      V.X := -V.X;
    end;
    if P.X >= W then
    begin
      P.X := W - 1;
      V.X := - V.X;
    end;
    if P.Y < 0 then
    begin
      P.Y := 0;
      V.Y := -V.Y;
    end;
    if P.Y >= H then
    begin
      P.Y := H - 1;
      V.Y := - V.Y
    end;

    { change velocity a little bit }
    V.X := V.X + (Random - 0.5) / 4;
    V.Y := V.Y + (Random - 0.5) / 4;

    { limit velocity }
    if V.X < -1 then V.X := -1;
    if V.X > 1 then V.X := 1;
    if V.Y < -1 then V.Y := -1;
    if V.Y > 1 then V.Y := 1;
  end;

begin
  AdvancePoint(P1, V1);
  AdvancePoint(P2, V2);
  Color := HSLtoRGB(ColorTime, 0.5, 0.5);
  Color := SetAlpha(Color, 40);
  ColorTime := ColorTime + Random / 300;
end;


{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  { set bitmap dimensions equal to image size }
  Image.SetupBitmap;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  if Thread1 = nil then
  begin
    { start the thread }
    Button1.Caption := 'Kill Thread #1';
    Thread1 := TPaintThread.Create(Image);
  end
  else
  begin
    { stop the thread }
    Thread1.Terminate;
    Thread1 := nil;
    Button1.Caption := 'Start Thread #1';
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  if Thread2 = nil then
  begin
    { start the thread }
    Button2.Caption := 'Kill Thread #2';
    Thread2 := TPaintThread.Create(Image);
  end
  else
  begin
    { stop the thread }
    Thread2.Terminate;
    Thread2 := nil;
    Button2.Caption := 'Start Thread #2';
  end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Thread1 <> nil then Thread1.Terminate;
  if Thread2 <> nil then Thread2.Terminate;
end;

procedure TForm1.ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  Image.Sprites[0].Position := Point(X - 48, Y - 32);
end;

end.
