/*

	Basic sample for using Gouraud shading.
	This sample also show how to build your world
	in real-time (without loading a pre-made world)
	The sample is well documented and cover many important
	issues so it is very recommended to read.
	The sample works only with a 3D graphics card.
	The card is initialized to work in full screen mode.

*/

#include "mrft_api.h"
DWORD create_a_polygon();


void main()
{
  DWORD polygon,point, camera;	
	
	//In most of Morfit samples we use Morfit_engine_load_world()
	//To load a world that was already built for us.
	//In this sample we will show how you can create your world in
	//realtime. Please note that the engine must be in EDITOR_MODE in order
	//to delete and add polygons at realtime.
	//The following line will set the engine to work in EDITOR_MODE
	Morfit_engine_load_world(NULL,NULL,NULL, EDITOR_MODE);

	//Now since we didnt load any world (we gave NULL) then we
	//will create our own world.
	polygon=create_a_polygon();
	if(polygon==NULL) {
		MessageBox(NULL, "Failed to create polygon. Please check morfit.log file", "Failed. Exiting", MB_OK);
		return;
	}

	camera=Morfit_camera_get_default_camera();
	//The default location is (0,0,0). We will change it to 200,0,0
	Morfit_camera_set_location(camera, 200,0 ,0);
	//Morfit_camera_set_direction(camera, -1,1 ,0);

	//Some general settings for beauty purposes
	Morfit_engine_set_atmospheric_effect(0,0,0); //set the background to black
	ShowCursor(FALSE);
	
	//Setting a full screen mode
	if(Morfit_3D_card_set_full_screen_mode(800,600)==VR_ERROR) {
		MessageBox(NULL, "Failed to initialize your 3D card.\n This sample only works with hardware rendering\nPlease check that you have a 3D card and DirectX 6.1 and above\n","No 3D card", MB_OK);
		return;
	}

	BYTE rgb[3]={0,0,0};
	int increase_flag, color_selector=0;
	//Lets render till we press the escape key on the keyboard.
	while( (GetAsyncKeyState(VK_ESCAPE)&1) ==0 ) {
		Morfit_engine_render(NULL,camera);	
		
		//Lets change each pixel rgb value
		for(point=Morfit_polygon_get_first_point(polygon); point!=NULL; point=Morfit_point_get_next_point(point) ) {
			Morfit_point_set_rgb(point,rgb);
		}
			
		if(rgb[color_selector]>=250) increase_flag=NO;
		if(rgb[color_selector]==0) {
			increase_flag=YES;
			color_selector++;
			if(color_selector>2) color_selector=0;
		}


		if(increase_flag==YES) rgb[color_selector]+=3;
		else rgb[color_selector]-=3;
		
		Sleep(1); //So it is not so fast ...

	}

}


//This function will create a square polygon
//
//
// 1*-------*2
//  |       |
//  |       |
//  |       |
// 4*-------*3
//
// Note that the four coords should be added in clockwise order.
// Note also that polygons have only one side.
//
// To understand more about the bitmap coords please check the
// book 3D Programing For Windows Using Morfit.
// In short (If you are too lazy ...) the first coord will be maped to
// the top left corner of the bitmap. u,v value of 0,0 corresponds to the
// top left corner of the bitmap. The third Coordinate will be mapped to
// the bottom right corner. u,v values of 1,1 denote the bottom right corner
// of the bitmap.
//
// Here are the bitmaps coord as they are mapped to the polygon
//
// (0,0)*-------*(1,0)
//      |       |
//      |       |
//      |       |
// (0,1)*-------*(1,1)
//
//
// Here are the points that define our square. Note that the X axis
// is going from the screen out. The Y axis is from left to right and
// the Z buffer is pointing up.
// 
// (0,-100,100) *-------*(0,100,100)
//              |       |
//				|       |
//				|       |
// (0,-100,-100)*-------*(0,100,-100)

DWORD create_a_polygon()
{
  double xyz_uv_b[6]; //The xyz of the point. The bitmaps cord Uand V and the brightness

	DWORD poly=Morfit_polygon_create();
	
	// Doing the first point (out of four)
	//------------------------------------
	//Setting the xyz coords
	xyz_uv_b[0]=0;
	xyz_uv_b[1]=-100;
	xyz_uv_b[2]=100;

	//Setting the bitmap coord
	xyz_uv_b[3]=0;
	xyz_uv_b[4]=0;

	xyz_uv_b[5]=100; //Normal brightness.
	
	Morfit_polygon_add_point(poly, xyz_uv_b);

	// Doing the second point
	//-----------------------
	//Setting the xyz coords
	xyz_uv_b[0]=0;
	xyz_uv_b[1]=100;
	xyz_uv_b[2]=100;

	//Setting the bitmap coord
	xyz_uv_b[3]=1;
	xyz_uv_b[4]=0;

	xyz_uv_b[5]=100; //Normal brightness.
	
	Morfit_polygon_add_point(poly, xyz_uv_b);


	// Doing the third point
	//-----------------------
	//Setting the xyz coords
	xyz_uv_b[0]=0;
	xyz_uv_b[1]=100;
	xyz_uv_b[2]=-100;

	//Setting the bitmap coord
	xyz_uv_b[3]=1;
	xyz_uv_b[4]=1;

	xyz_uv_b[5]=100; //Normal brightness.
	
	Morfit_polygon_add_point(poly, xyz_uv_b);

	// Doing the fourth point
	//-----------------------
	//Setting the xyz coords
	xyz_uv_b[0]=0;
	xyz_uv_b[1]=-100;
	xyz_uv_b[2]=-100;

	//Setting the bitmap coord
	xyz_uv_b[3]=0;
	xyz_uv_b[4]=1;

	xyz_uv_b[5]=100; //Normal brightness.
	
	Morfit_polygon_add_point(poly, xyz_uv_b);

	DWORD bmp=Morfit_bitmap_load("flare2",-1);
	Morfit_polygon_set_bitmap_fill(poly, bmp);

	if(Morfit_engine_add_polygon(poly)==VR_ERROR) return(NULL);
	
	return(poly);

}