Unit oaBitmap;

{ Assistant Package v1.5 }
{ Copyright  1999 Kambiz R. Khojasteh }
{ email: Khojasteh@www.dci.co.ir }

{ Thanks to Roberto Cardentey Ruiz for solving the transparency }
{ problem on Windows NT. }
{ email: robe@electrica.ispjae.edu.cu }

{$IFDEF VER100}
  {$DEFINE OA_D3}
{$ENDIF}

interface

uses
  WinTypes, WinProcs, Classes, Graphics;

{$IFDEF OA_D3}
procedure StretchDrawBitmapTransparent(Canvas: TCanvas; Rect: TRect;
  Bitmap: TBitmap; TransparentColor: TColor);
procedure DrawBitmapTransparent(Canvas: TCanvas; X, Y: Integer;
  Bitmap: TBitmap; TransparentColor: TColor);
{$ENDIF}

function CreateBitmapRgn(Bitmap: TBitmap; TransparentColor: TColor): HRgn;

implementation

uses
  SysUtils;

{$IFDEF OA_D3}

// This procedure is copied from RxLibrary VCLUtils.
function PaletteColor(Color: TColor): Longint;
begin
  Result := ColorToRGB(Color) or $02000000;
end;

// This procedure is copied from RxLibrary VCLUtils.
procedure StretchBltTransparent(DstDC: HDC; DstX, DstY, DstW, DstH: Integer;
  SrcDC: HDC; SrcX, SrcY, SrcW, SrcH: Integer; Palette: HPalette;
  TransparentColor: TColorRef);
var
  Color: TColorRef;
  bmAndBack, bmAndObject, bmAndMem, bmSave: HBitmap;
  bmBackOld, bmObjectOld, bmMemOld, bmSaveOld: HBitmap;
  MemDC, BackDC, ObjectDC, SaveDC: HDC;
  palDst, palMem, palSave, palObj: HPalette;
begin
  { Create some DCs to hold temporary data }
  BackDC := CreateCompatibleDC(DstDC);
  ObjectDC := CreateCompatibleDC(DstDC);
  MemDC := CreateCompatibleDC(DstDC);
  SaveDC := CreateCompatibleDC(DstDC);
  { Create a bitmap for each DC }
  bmAndObject := CreateBitmap(SrcW, SrcH, 1, 1, nil);
  bmAndBack := CreateBitmap(SrcW, SrcH, 1, 1, nil);
  bmAndMem := CreateCompatibleBitmap(DstDC, DstW, DstH);
  bmSave := CreateCompatibleBitmap(DstDC, SrcW, SrcH);
  { Each DC must select a bitmap object to store pixel data }
  bmBackOld := SelectObject(BackDC, bmAndBack);
  bmObjectOld := SelectObject(ObjectDC, bmAndObject);
  bmMemOld := SelectObject(MemDC, bmAndMem);
  bmSaveOld := SelectObject(SaveDC, bmSave);
  { Select palette }
  palDst := 0; palMem := 0; palSave := 0; palObj := 0;
  if Palette <> 0 then begin
    palDst := SelectPalette(DstDC, Palette, True);
    RealizePalette(DstDC);
    palSave := SelectPalette(SaveDC, Palette, False);
    RealizePalette(SaveDC);
    palObj := SelectPalette(ObjectDC, Palette, False);
    RealizePalette(ObjectDC);
    palMem := SelectPalette(MemDC, Palette, True);
    RealizePalette(MemDC);
  end;
  { Set proper mapping mode }
  SetMapMode(SrcDC, GetMapMode(DstDC));
  SetMapMode(SaveDC, GetMapMode(DstDC));
  { Save the bitmap sent here }
  BitBlt(SaveDC, 0, 0, SrcW, SrcH, SrcDC, SrcX, SrcY, SRCCOPY);
  { Set the background color of the source DC to the color,         }
  { contained in the parts of the bitmap that should be transparent }
  Color := SetBkColor(SaveDC, PaletteColor(TransparentColor));
  { Create the object mask for the bitmap by performing a BitBlt()  }
  { from the source bitmap to a monochrome bitmap                   }
  BitBlt(ObjectDC, 0, 0, SrcW, SrcH, SaveDC, 0, 0, SRCCOPY);
  { Set the background color of the source DC back to the original  }
  SetBkColor(SaveDC, Color);
  { Create the inverse of the object mask }
  BitBlt(BackDC, 0, 0, SrcW, SrcH, ObjectDC, 0, 0, NOTSRCCOPY);
  { Copy the background of the main DC to the destination }
  BitBlt(MemDC, 0, 0, DstW, DstH, DstDC, DstX, DstY, SRCCOPY);
  { Mask out the places where the bitmap will be placed }
  StretchBlt(MemDC, 0, 0, DstW, DstH, ObjectDC, 0, 0, SrcW, SrcH, SRCAND);
  { Mask out the transparent colored pixels on the bitmap }
  BitBlt(SaveDC, 0, 0, SrcW, SrcH, BackDC, 0, 0, SRCAND);
  { XOR the bitmap with the background on the destination DC }
  StretchBlt(MemDC, 0, 0, DstW, DstH, SaveDC, 0, 0, SrcW, SrcH, SRCPAINT);
  { Copy the destination to the screen }
  BitBlt(DstDC, DstX, DstY, DstW, DstH, MemDC, 0, 0,
    SRCCOPY);
  { Restore palette }
  if Palette <> 0 then begin
    SelectPalette(MemDC, palMem, False);
    SelectPalette(ObjectDC, palObj, False);
    SelectPalette(SaveDC, palSave, False);
    SelectPalette(DstDC, palDst, True);
  end;
  { Delete the memory bitmaps }
  DeleteObject(SelectObject(BackDC, bmBackOld));
  DeleteObject(SelectObject(ObjectDC, bmObjectOld));
  DeleteObject(SelectObject(MemDC, bmMemOld));
  DeleteObject(SelectObject(SaveDC, bmSaveOld));
  { Delete the memory DCs }
  DeleteDC(MemDC);
  DeleteDC(BackDC);
  DeleteDC(ObjectDC);
  DeleteDC(SaveDC);
end;

// This procedure is copied from RxLibrary VCLUtils.
procedure StretchBitmapTransparent(Dest: TCanvas; Bitmap: TBitmap;
  TransparentColor: TColor; DstX, DstY, DstW, DstH, SrcX, SrcY,
  SrcW, SrcH: Integer);
var
  CanvasChanging: TNotifyEvent;
begin
  if DstW <= 0 then DstW := Bitmap.Width;
  if DstH <= 0 then DstH := Bitmap.Height;
  if (SrcW <= 0) or (SrcH <= 0)
  then begin
    SrcX := 0; SrcY := 0;
    SrcW := Bitmap.Width;
    SrcH := Bitmap.Height;
  end;
  if not Bitmap.Monochrome then
    SetStretchBltMode(Dest.Handle, STRETCH_DELETESCANS);
  CanvasChanging := Bitmap.Canvas.OnChanging;
  Bitmap.Canvas.Lock;
  try
    Bitmap.Canvas.OnChanging := nil;
    if TransparentColor = clNone then begin
      StretchBlt(Dest.Handle, DstX, DstY, DstW, DstH, Bitmap.Canvas.Handle,
        SrcX, SrcY, SrcW, SrcH, Dest.CopyMode);
    end
    else
    begin
      if TransparentColor = clDefault then
        TransparentColor := Bitmap.Canvas.Pixels[0, Bitmap.Height - 1];
      if Bitmap.Monochrome then TransparentColor := clWhite
      else TransparentColor := ColorToRGB(TransparentColor);
      StretchBltTransparent(Dest.Handle, DstX, DstY, DstW, DstH,
        Bitmap.Canvas.Handle, SrcX, SrcY, SrcW, SrcH, Bitmap.Palette,
        TransparentColor);
    end;
  finally
    Bitmap.Canvas.OnChanging := CanvasChanging;
    Bitmap.Canvas.Unlock;
  end;
end;

procedure StretchDrawBitmapTransparent(Canvas: TCanvas; Rect: TRect;
  Bitmap: TBitmap; TransparentColor: TColor);
begin
  StretchBitmapTransparent(Canvas, Bitmap, TransparentColor,
    Rect.Left, Rect.Top, Rect.Right - Rect.Left, Rect.Bottom - Rect.Top,
    0, 0, Bitmap.Width-1, Bitmap.Height-1);
end;

procedure DrawBitmapTransparent(Canvas: TCanvas; X, Y: Integer;
  Bitmap: TBitmap; TransparentColor: TColor);
begin
  StretchBitmapTransparent(Canvas, Bitmap, TransparentColor, X, Y,
    Bitmap.Width, Bitmap.Height, 0, 0, Bitmap.Width, Bitmap.Height);
end;

{$ENDIF}

function CreateBitmapRgn(Bitmap: TBitmap; TransparentColor: TColor): HRgn;
var
  BitmapMask: TBitmap;
  RectList: TList;
  RgnData: PRgnData;
  I, Size: Integer;
  Rects: PRect;

  procedure FindRegionRectangles;
  var
    Mask: Byte;
    B: PByteArray;
    X, Y, I, Left, Right: Integer;
    PR: PRect;
  begin
    for Y := 0 to BitmapMask.Height-1 do
    begin
      Left := 0;
      Right := -1;
      I := 0;
      Mask := $80;
      B := BitmapMask.ScanLine[Y];
      for X := 0 to BitmapMask.Width-1 do
      begin
        if (Mask and B[I]) <> 0 then
        begin
          if Right >= Left then
          begin
            New(PR);
            PR^.Left := Left;
            PR^.Top := Y;
            PR^.Right := Right+1;
            PR^.Bottom := Y+1;
            RectList.Add(PR);
          end;
          Left := Right+2;
        end;
        Inc(Right);
        Mask := Mask shr 1;
        if Mask = 0 then
        begin
          Inc(I);
          Mask := $80;
        end;
      end;
      if Right >= Left then
      begin
        New(PR);
        PR^.Left := Left;
        PR^.Top := Y;
        PR^.Right := Right+1;
        PR^.Bottom := Y+1;
        RectList.Add(PR);
      end;
    end;
  end;

begin
  RectList := TList.Create;
  BitmapMask := TBitmap.Create;
  try
    BitmapMask.Assign(Bitmap);
    BitmapMask.Mask(TransparentColor);
    BitmapMask.PixelFormat := pf1bit;
    FindRegionRectangles;
    Size := SizeOf(TRgnData) + (RectList.Count * SizeOf(TRect));
    GetMem(RgnData, Size);
    try
      with RgnData^.rdh do
      begin
        dwSize := SizeOf(TRgnDataHeader);
        iType := RDH_RECTANGLES;
        nCount := RectList.Count;
        nRgnSize := SizeOf(TRect);
        rcBound := Rect(0, 0, Bitmap.Width-1, Bitmap.Height-1);
      end;
      Rects := PRect(@RgnData^.Buffer);
      for I := 0 to RectList.Count-1 do
      begin
        Rects^ := PRect(RectList.Items[I])^;
        Dispose(PRect(RectList.Items[I]));
        Inc(Rects);
      end;
      Result := ExtCreateRegion(nil, Size, RgnData^)
    finally
      FreeMem(RgnData, Size);
    end;
  finally
    BitmapMask.Free;
    RectList.Free;
  end;
end;

{ This function is slower than previous function
function CreateBitmapRgn(Bitmap: TBitmap; TransparentColor: TColor): HRgn;
var
  X, Y, I, Left, Right: Integer;
  BitmapMask: TBitmap;
  B: PByteArray;
  Mask: Byte;
  Rgn: HRgn;
begin
  Result := NULLREGION;
  BitmapMask := TBitmap.Create;
  try
    BitmapMask.Assign(Bitmap);
    BitmapMask.Mask(TransparentColor);
    BitmapMask.PixelFormat := pf1bit;
    for Y := 0 to BitmapMask.Height-1 do
    begin
      Left := 0;
      Right := -1;
      I := 0;
      Mask := $80;
      B := BitmapMask.ScanLine[Y];
      for X := 0 to BitmapMask.Width-1 do
      begin
        if (Mask and B[I]) <> 0 then
        begin
          if Right >= Left then
          begin
            Rgn := CreateRectRgn(Left, Y, Right+1, Y+1);
            if Result <> NULLREGION then
            begin
              CombineRgn(Result, Result, Rgn, RGN_OR);
              DeleteObject(Rgn);
            end
            else
              Result := Rgn;
          end;
          Left := Right+2;
        end;
        Inc(Right);
        Mask := Mask shr 1;
        if Mask = 0 then
        begin
          Inc(I);
          Mask := $80;
        end;
      end;
      if Right >= Left then
      begin
        Rgn := CreateRectRgn(Left, Y, Right+1, Y+1);
        if Result <> NULLREGION then
        begin
          CombineRgn(Result, Result, Rgn, RGN_OR);
          DeleteObject(Rgn);
        end
        else
          Result := Rgn;
      end;
    end;
  finally
    BitmapMask.Free;
  end;
end;
}

end.
